"""
********************************************************************************
compas_rcf.utils
********************************************************************************

.. currentmodule:: compas_rcf.utils

RhinoCommon to COMPAS object conversions
----------------------------------------
Scripts for converting geometry objects between the two frameworks. Most of this
has been integrated into COMPAS v0.15.0 but compas_fab needs earlier versions
of COMPAS.

JSON tools
----------
Utility functions for loading and dumping
:class:`compas_rcf.fabrication.ClayBullet` objects.

Assorted utilities
------------------
When there's no other good place for a bit of code..


"""
