"""
********************************************************************************
compas_rcf.fabrication
********************************************************************************

.. currentmodule:: compas_rcf.fabrication

Runners & fabrication objects
-----------------------------
This modules contains runners; scripts for running a fabrication process using a
digital fabrication method. It also contains fabrication objects, most significantly
:class:`compas_rcf.fabrication.ClayBullet` which is our data representation of our
discrete building blocks.

"""
